@@ -1,33 +1,82 @@
 """
 Provides tests for the syncing functionalities in django entity.
 """
-from mock import patch
-
 from django.contrib.contenttypes.models import ContentType
 from django.core.management import call_command
-from django.db.models.signals import post_delete, post_save
 from django.test.utils import override_settings
-from entity.models import Entity, EntityRelationship, delete_entity_signal_handler, save_entity_signal_handler
-from entity.sync import sync_entities, turn_on_syncing, turn_off_syncing
+from django_dynamic_fixture import G, F
+from entity.config import EntityRegistry
+from entity.models import Entity, EntityRelationship
+from entity.sync import EntitySyncer
+from entity import turn_on_syncing, turn_off_syncing, sync_entities
+from mock import patch
 
-from .models import Account, Team, EntityPointer, DummyModel, MultiInheritEntity
-from .utils import EntityTestCase
+from entity.tests.models import (
+    Account, Team, EntityPointer, DummyModel, MultiInheritEntity, AccountConfig, TeamConfig, TeamGroup,
+    M2mEntity, PointsToM2mEntity, PointsToAccount, Competitor
+)
+from entity.tests.utils import EntityTestCase
 
 
 class TestTurnOnOffSyncing(EntityTestCase):
     """
     Tests turning on and off entity syncing.
     """
-    def tearDown(self):
-        super(TestTurnOnOffSyncing, self).tearDown()
-        # Make sure syncing is always on before every test
-        turn_on_syncing()
+    @patch('entity.models.post_save', spec_set=True)
+    @patch('entity.models.post_delete', spec_set=True)
+    @patch('entity.models.m2m_changed', spec_set=True)
+    @patch('entity.models.post_bulk_operation', spec_set=True)
+    def test_turn_on_syncing_all_handlers_true(
+            self, post_bulk_operation_mock, m2m_changed_mock, post_delete_mock, post_save_mock):
+        turn_on_syncing(for_post_save=True, for_post_delete=True, for_m2m_changed=True, for_post_bulk_operation=True)
+        self.assertTrue(post_save_mock.connect.called)
+        self.assertTrue(post_delete_mock.connect.called)
+        self.assertTrue(m2m_changed_mock.connect.called)
+        self.assertTrue(post_bulk_operation_mock.connect.called)
+
+    @patch('entity.models.post_save', spec_set=True)
+    @patch('entity.models.post_delete', spec_set=True)
+    @patch('entity.models.m2m_changed', spec_set=True)
+    @patch('entity.models.post_bulk_operation', spec_set=True)
+    def test_turn_on_syncing_all_handlers_false(
+            self, post_bulk_operation_mock, m2m_changed_mock, post_delete_mock, post_save_mock):
+        turn_on_syncing(
+            for_post_save=False, for_post_delete=False, for_m2m_changed=False, for_post_bulk_operation=False)
+        self.assertFalse(post_save_mock.connect.called)
+        self.assertFalse(post_delete_mock.connect.called)
+        self.assertFalse(m2m_changed_mock.connect.called)
+        self.assertFalse(post_bulk_operation_mock.connect.called)
+
+    @patch('entity.models.post_save', spec_set=True)
+    @patch('entity.models.post_delete', spec_set=True)
+    @patch('entity.models.m2m_changed', spec_set=True)
+    @patch('entity.models.post_bulk_operation', spec_set=True)
+    def test_turn_off_syncing_all_handlers_true(
+            self, post_bulk_operation_mock, m2m_changed_mock, post_delete_mock, post_save_mock):
+        turn_off_syncing(for_post_save=True, for_post_delete=True, for_m2m_changed=True, for_post_bulk_operation=True)
+        self.assertTrue(post_save_mock.disconnect.called)
+        self.assertTrue(post_delete_mock.disconnect.called)
+        self.assertTrue(m2m_changed_mock.disconnect.called)
+        self.assertTrue(post_bulk_operation_mock.disconnect.called)
+
+    @patch('entity.models.post_save', spec_set=True)
+    @patch('entity.models.post_delete', spec_set=True)
+    @patch('entity.models.m2m_changed', spec_set=True)
+    @patch('entity.models.post_bulk_operation', spec_set=True)
+    def test_turn_off_syncing_all_handlers_false(
+            self, post_bulk_operation_mock, m2m_changed_mock, post_delete_mock, post_save_mock):
+        turn_off_syncing(
+            for_post_save=False, for_post_delete=False, for_m2m_changed=False, for_post_bulk_operation=False)
+        self.assertFalse(post_save_mock.disconnect.called)
+        self.assertFalse(post_delete_mock.disconnect.called)
+        self.assertFalse(m2m_changed_mock.disconnect.called)
+        self.assertFalse(post_bulk_operation_mock.disconnect.called)
 
     def test_post_save_turned_on_by_default(self):
         """
         Tests that save signals are connected by default.
         """
-        with patch('entity.models.sync_entity_signal_handler') as mock_handler:
+        with patch('entity.models.sync_entities') as mock_handler:
             Account.objects.create()
             self.assertTrue(mock_handler.called)
 
@@ -35,28 +84,26 @@ class TestTurnOnOffSyncing(EntityTestCase):
         """
         Tests that delete signals are connected by default.
         """
-        with patch('entity.models.sync_entity_signal_handler') as mock_handler:
-            a = Account.objects.create()
-            self.assertEquals(mock_handler.call_count, 1)
-
+        a = Account.objects.create()
+        with patch('entity.models.Entity.objects.delete_for_obj') as mock_handler:
             # Delete the object. The signal should be called
             a.delete()
-            self.assertEquals(mock_handler.call_count, 2)
+            self.assertEquals(mock_handler.call_count, 1)
 
-    def test_bulk_operation_turned_on_by_default(self):
+    def test_bulk_operation_turned_off_by_default(self):
         """
-        Tests that bulk operations are turned on by default.
+        Tests that bulk operations are turned off by default.
         """
-        with patch('entity.models.sync_entities_signal_handler') as mock_handler:
+        with patch('entity.models.sync_entities') as mock_handler:
             Account.objects.bulk_create([Account() for i in range(5)])
-            self.assertTrue(mock_handler.called)
+            self.assertFalse(mock_handler.called)
 
     def test_turn_off_save(self):
         """
         Tests turning off syncing for the save signal.
         """
         turn_off_syncing()
-        with patch('entity.models.sync_entity_signal_handler') as mock_handler:
+        with patch('entity.models.sync_entities') as mock_handler:
             Account.objects.create()
             self.assertFalse(mock_handler.called)
 
@@ -65,7 +112,7 @@ class TestTurnOnOffSyncing(EntityTestCase):
         Tests turning off syncing for the delete signal.
         """
         turn_off_syncing()
-        with patch('entity.models.sync_entity_signal_handler') as mock_handler:
+        with patch('entity.models.sync_entities') as mock_handler:
             a = Account.objects.create()
             self.assertFalse(mock_handler.called)
             a.delete()
@@ -76,7 +123,7 @@ class TestTurnOnOffSyncing(EntityTestCase):
         Tests turning off syncing for bulk operations.
         """
         turn_off_syncing()
-        with patch('entity.models.sync_entities_signal_handler') as mock_handler:
+        with patch('entity.models.sync_entities') as mock_handler:
             Account.objects.bulk_create([Account() for i in range(5)])
             self.assertFalse(mock_handler.called)
 
@@ -86,7 +133,7 @@ class TestTurnOnOffSyncing(EntityTestCase):
         """
         turn_off_syncing()
         turn_on_syncing()
-        with patch('entity.models.sync_entity_signal_handler') as mock_handler:
+        with patch('entity.models.sync_entities') as mock_handler:
             Account.objects.create()
             self.assertTrue(mock_handler.called)
 
@@ -96,47 +143,36 @@ class TestTurnOnOffSyncing(EntityTestCase):
         """
         turn_off_syncing()
         turn_on_syncing()
-        with patch('entity.models.sync_entity_signal_handler') as mock_handler:
+        with patch('entity.models.Entity.objects.delete_for_obj') as mock_handler:
             a = Account.objects.create()
-            self.assertEquals(mock_handler.call_count, 1)
             a.delete()
-            self.assertEquals(mock_handler.call_count, 2)
+            self.assertEquals(mock_handler.call_count, 1)
 
     def test_turn_on_bulk(self):
         """
         Tests turning on syncing for bulk operations.
         """
         turn_off_syncing()
-        turn_on_syncing()
-        with patch('entity.models.sync_entities_signal_handler') as mock_handler:
+        turn_on_syncing(for_post_bulk_operation=True)
+        with patch('entity.models.sync_entities') as mock_handler:
             Account.objects.bulk_create([Account() for i in range(5)])
             self.assertTrue(mock_handler.called)
 
 
-class TestSyncAllEntities(EntityTestCase):
+class SyncAllEntitiesTest(EntityTestCase):
     """
     Tests that all entities can be synced at once and tests the management command to
     sync all entities.
     """
-    def setUp(self):
-        super(TestSyncAllEntities, self).setUp()
-        # Disconnect signal handlers to test syncing all entities
-        post_delete.disconnect(delete_entity_signal_handler, dispatch_uid='delete_entity_signal_handler')
-        post_save.disconnect(save_entity_signal_handler, dispatch_uid='save_entity_signal_handler')
-
-    def tearDown(self):
-        super(TestSyncAllEntities, self).tearDown()
-        # Reconnect signal handlers
-        post_delete.connect(delete_entity_signal_handler, dispatch_uid='delete_entity_signal_handler')
-        post_save.connect(save_entity_signal_handler, dispatch_uid='save_entity_signal_handler')
-
     def test_sync_entities_management_command(self):
         """
         Tests that the management command for syncing entities works properly.
         """
         # Create five test accounts
+        turn_off_syncing()
         for i in range(5):
             Account.objects.create()
+        turn_on_syncing()
 
         # Test that the management command syncs all five entities
         self.assertEquals(Entity.objects.all().count(), 0)
@@ -148,9 +184,11 @@ class TestSyncAllEntities(EntityTestCase):
         """
         Tests that the sync_entities command works with the asynchronous option.
         """
-        # Create five test accounts
+        # Create five test accounts without syncing on
+        turn_off_syncing()
         for i in range(5):
             Account.objects.create()
+        turn_on_syncing()
 
         # Test that the management command syncs all five entities
         self.assertEquals(Entity.objects.all().count(), 0)
@@ -182,8 +220,10 @@ class TestSyncAllEntities(EntityTestCase):
         """
         Tests syncing all accounts with no super entities.
         """
+        turn_off_syncing()
         # Create five test accounts
         accounts = [Account.objects.create() for i in range(5)]
+        turn_on_syncing()
 
         # Sync all of the entities and verify that five Entity models were created for the Account model
         self.assertEquals(Entity.objects.all().count(), 0)
@@ -192,7 +232,10 @@ class TestSyncAllEntities(EntityTestCase):
 
         # Delete an account. When all entities are synced again,
         # there should only be four accounts
+        turn_off_syncing()
         accounts[0].delete()
+        turn_on_syncing()
+
         self.assertEquals(Entity.objects.all().count(), 5)
         sync_entities()
         self.assertEquals(Entity.objects.all().count(), 4)
@@ -223,32 +266,15 @@ class TestSyncAllEntities(EntityTestCase):
         # There should be four entity relationships since four accounts have teams
         self.assertEquals(EntityRelationship.objects.all().count(), 4)
 
-    def test_sync_optimal_queries(self):
-        """
-        Tests optimal queries of syncing.
-        """
-        # Create five test accounts
-        accounts = [Account.objects.create() for i in range(5)]
-        # Create two teams to assign to some of the accounts
-        teams = [Team.objects.create() for i in range(2)]
-        accounts[0].team = teams[0]
-        accounts[0].save()
-        accounts[1].team = teams[0]
-        accounts[1].save()
-        accounts[2].team = teams[1]
-        accounts[2].save()
-        accounts[3].team = teams[1]
-        accounts[3].save()
 
-        with self.assertNumQueries(56):
-            sync_entities()
-
-
-class TestEntitySignalSync(EntityTestCase):
+class TestEntityBulkSignalSync(EntityTestCase):
     """
-    Tests that entities (from the test models) are properly synced upon post_save and
-    post_delete calls.
+    Tests syncing when bulk operations happen.
     """
+    def setUp(self):
+        super(TestEntityBulkSignalSync, self).setUp()
+        turn_on_syncing(for_post_bulk_operation=True)
+
     def test_post_bulk_create(self):
         """
         Tests that entities can have bulk creates applied to them and still be synced.
@@ -280,6 +306,13 @@ class TestEntitySignalSync(EntityTestCase):
             self.assertEquals(entity.entity_meta['email'], 'test2@test.com')
         self.assertEquals(Entity.objects.all().count(), 5)
 
+    def test_invalid_entity_model(self):
+        """
+        Tests that an invalid entity model is not synced on bulk update.
+        """
+        DummyModel.objects.bulk_create([DummyModel()])
+        self.assertFalse(Entity.objects.exists())
+
     def test_post_bulk_update_dummy(self):
         """
         Tests that even if the dummy model is using the special model manager for bulk
@@ -290,6 +323,142 @@ class TestEntitySignalSync(EntityTestCase):
         # There should be no synced entities
         self.assertEquals(Entity.objects.all().count(), 0)
 
+
+class TestWatching(EntityTestCase):
+    """
+    Tests when an entity is watching another model for changes.
+    """
+    def test_m2m_changed_of_another_model(self):
+        """
+        Tests when an entity model is listening for a change of an m2m of another model.
+        """
+        m2m_entity = G(M2mEntity)
+        team = G(Team)
+        points_to_m2m_entity = G(PointsToM2mEntity, m2m_entity=m2m_entity)
+        # Three entities should be synced and there should not yet be any relationships
+        self.assertEquals(Entity.objects.count(), 3)
+        self.assertFalse(EntityRelationship.objects.exists())
+
+        # When a team is added to the m2m entity, it should be a super entity to the points_to_m2m_entity and
+        # of m2m_entity
+        m2m_entity.teams.add(team)
+        self.assertEquals(Entity.objects.count(), 3)
+        self.assertEquals(EntityRelationship.objects.count(), 2)
+
+        points_to_m2m_entity = Entity.objects.get_for_obj(points_to_m2m_entity)
+        team_entity = Entity.objects.get_for_obj(team)
+        m2m_entity = Entity.objects.get_for_obj(m2m_entity)
+        self.assertTrue(EntityRelationship.objects.filter(
+            sub_entity=points_to_m2m_entity, super_entity=team_entity).exists())
+        self.assertTrue(EntityRelationship.objects.filter(sub_entity=m2m_entity, super_entity=team_entity).exists())
+
+    def test_points_to_account_config_competitor_updated(self):
+        """
+        Tests that a PointsToAccount model is updated when the competitor of its account is updated.
+        """
+        account = G(Account)
+        pta = G(PointsToAccount, account=account)
+        pta_entity = Entity.objects.get_for_obj(pta)
+        self.assertEquals(pta_entity.entity_meta, {
+            'team_name': 'None',
+            'competitor_name': 'None',
+        })
+
+        team = G(Team, name='team1')
+        competitor = G(Competitor, name='competitor1')
+        account.team = team
+        account.competitor = competitor
+        account.save()
+
+        # Nothing should have been updated on the entity. This is because it is watching the competitor
+        # and team models for changes. Since these models were changed before they were linked to the
+        # account, the changes are not propagated.
+        pta_entity = Entity.objects.get_for_obj(pta)
+        self.assertEquals(pta_entity.entity_meta, {
+            'team_name': 'None',
+            'competitor_name': 'None',
+        })
+
+        # Now change names of the competitors and teams. Things will be propagated.
+        team.name = 'team2'
+        team.save()
+        pta_entity = Entity.objects.get_for_obj(pta)
+        self.assertEquals(pta_entity.entity_meta, {
+            'team_name': 'team2',
+            'competitor_name': 'competitor1',
+        })
+
+        competitor.name = 'competitor2'
+        competitor.save()
+        pta_entity = Entity.objects.get_for_obj(pta)
+        self.assertEquals(pta_entity.entity_meta, {
+            'team_name': 'team2',
+            'competitor_name': 'competitor2',
+        })
+
+        # The power of django entity compels you...
+
+
+class TestEntityM2mChangedSignalSync(EntityTestCase):
+    """
+    Tests when an m2m changes on a synced entity.
+    """
+    def test_save_model_with_m2m(self):
+        """
+        Verifies that the m2m test entity is synced properly upon save.
+        """
+        turn_off_syncing()
+        m = G(M2mEntity)
+        m.teams.add(G(Team))
+        turn_on_syncing()
+
+        m.save()
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 1)
+
+    def test_sync_when_m2m_add(self):
+        """
+        Verifies an entity is synced properly when and m2m field is added.
+        """
+        m = G(M2mEntity)
+        self.assertEquals(Entity.objects.count(), 1)
+        self.assertEquals(EntityRelationship.objects.count(), 0)
+        m.teams.add(G(Team))
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 1)
+
+    def test_sync_when_m2m_delete(self):
+        """
+        Verifies an entity is synced properly when and m2m field is deleted.
+        """
+        m = G(M2mEntity)
+        team = G(Team)
+        m.teams.add(team)
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 1)
+        m.teams.remove(team)
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 0)
+
+    def test_sync_when_m2m_clear(self):
+        """
+        Verifies an entity is synced properly when and m2m field is cleared.
+        """
+        m = G(M2mEntity)
+        team = G(Team)
+        m.teams.add(team)
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 1)
+        m.teams.clear()
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 0)
+
+
+class TestEntityPostSavePostDeleteSignalSync(EntityTestCase):
+    """
+    Tests that entities (from the test models) are properly synced upon post_save
+    and post_delete signals.
+    """
     def test_post_save_dummy_data(self):
         """
         Tests that dummy data that does not inherit from EntityModelMixin is not synced
@@ -376,6 +545,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'test@test.com',
             'is_captain': False,
             'team': None,
+            'team_is_active': None,
         })
         self.assertEquals(entity.is_active, True)
 
@@ -402,6 +572,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'test@test.com',
             'is_captain': False,
             'team': 'Team',
+            'team_is_active': True,
         })
         team_entity = Entity.objects.get_for_obj(team)
         self.assertEquals(team_entity.entity_meta, None)
@@ -424,6 +595,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'test@test.com',
             'is_captain': False,
             'team': None,
+            'team_is_active': None,
         })
         old_entity_id = entity.id
 
@@ -440,6 +612,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'newemail@test.com',
             'is_captain': False,
             'team': None,
+            'team_is_active': None,
         })
         self.assertEquals(old_entity_id, entity.id)
 
@@ -458,6 +631,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'test@test.com',
             'is_captain': False,
             'team': None,
+            'team_is_active': None,
         })
 
         # Update the account's metadata and check that it is mirrored
@@ -468,6 +642,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'newemail@test.com',
             'is_captain': False,
             'team': None,
+            'team_is_active': None,
         })
 
     def test_post_update_account_relationship_activity(self):
@@ -490,6 +665,7 @@ class TestEntitySignalSync(EntityTestCase):
             'email': 'test@test.com',
             'is_captain': False,
             'team': 'Team',
+            'team_is_active': True,
         })
         team_entity = Entity.objects.get_for_obj(team)
         self.assertEquals(team_entity.entity_meta, None)
@@ -510,3 +686,123 @@ class TestEntitySignalSync(EntityTestCase):
         relationship = EntityRelationship.objects.first()
         self.assertEquals(relationship.sub_entity, account_entity)
         self.assertEquals(relationship.super_entity, team_entity)
+
+
+class TestSyncingMultipleEntities(EntityTestCase):
+    """
+    Tests syncing multiple entities at once of different types.
+    """
+    def test_sync_two_accounts(self):
+        turn_off_syncing()
+        team = G(Team)
+        account1 = G(Account, team=team)
+        account2 = G(Account, team=team)
+        G(TeamGroup)
+        sync_entities(account1, account2)
+
+        self.assertEquals(Entity.objects.count(), 3)
+        self.assertEquals(EntityRelationship.objects.count(), 2)
+
+    def test_sync_two_accounts_one_team_group(self):
+        turn_off_syncing()
+        team = G(Team)
+        account1 = G(Account, team=team)
+        account2 = G(Account, team=team)
+        team_group = G(TeamGroup)
+        sync_entities(account1, account2, team_group)
+
+        self.assertEquals(Entity.objects.count(), 4)
+        self.assertEquals(EntityRelationship.objects.count(), 2)
+
+
+class TestCachingAndCascading(EntityTestCase):
+    """
+    Tests caching, cascade syncing, and optimal queries when syncing single, multiple, or all entities.
+    """
+    def test_cascade_sync_super_entities(self):
+        """
+        Tests that super entities will be synced when a sub entity is synced (even if the super entities
+        werent synced before)
+        """
+        turn_off_syncing()
+        team = G(Team)
+        turn_on_syncing()
+
+        self.assertFalse(Entity.objects.exists())
+        G(Account, team=team)
+        self.assertEquals(Entity.objects.count(), 2)
+        self.assertEquals(EntityRelationship.objects.count(), 1)
+
+    def test_no_cascade_if_super_entity_exists(self):
+        """
+        Tests that super entities arent synced again if they have already been synced.
+        """
+        account = G(Account, team=F(team_group=F()))
+        self.assertTrue(Account.objects.exists())
+        self.assertTrue(Team.objects.exists())
+        self.assertTrue(TeamGroup.objects.exists())
+
+        entity_syncer = EntitySyncer()
+        entity_syncer.sync_entities_and_relationships(account)
+        # Verify that only the account and team reside in the entity syncer cache. This means that
+        # the syncing didnt percolate all the way to the team group
+        self.assertEquals(len(entity_syncer._synced_entity_cache), 2)
+
+    def test_optimal_queries_registered_entity_with_no_qset(self):
+        """
+        Tests that the optimal number of queries are performed when syncing a single entity that
+        did not register a queryset.
+        """
+        team_group = G(TeamGroup)
+
+        ContentType.objects.clear_cache()
+        with self.assertNumQueries(4):
+            team_group.save()
+
+    def test_optimal_queries_registered_entity_w_qset(self):
+        """
+        Tests that the entity is refetch with its queryset when syncing an individual entity.
+        """
+        account = G(Account)
+
+        ContentType.objects.clear_cache()
+        with self.assertNumQueries(5):
+            account.save()
+
+    def test_sync_all_optimal_queries(self):
+        """
+        Tests optimal queries of syncing all entities.
+        """
+        # Create five test accounts
+        accounts = [Account.objects.create() for i in range(5)]
+        # Create two teams to assign to some of the accounts
+        teams = [Team.objects.create() for i in range(2)]
+        accounts[0].team = teams[0]
+        accounts[0].save()
+        accounts[1].team = teams[0]
+        accounts[1].save()
+        accounts[2].team = teams[1]
+        accounts[2].save()
+        accounts[3].team = teams[1]
+        accounts[3].save()
+
+        # Use an entity registry that only has accounts and teams. This ensures that other registered
+        # entity models dont pollute the test case
+        new_registry = EntityRegistry()
+        new_registry.register_entity(
+            Account.objects.select_related('team', 'team2', 'team_group', 'competitor'), AccountConfig)
+        new_registry.register_entity(
+            Team.objects.select_related('team_group'), TeamConfig)
+
+        with patch('entity.sync.entity_registry') as mock_entity_registry:
+            mock_entity_registry.entity_registry = new_registry.entity_registry
+            ContentType.objects.clear_cache()
+            with self.assertNumQueries(14):
+                sync_entities()
+
+        self.assertEquals(Entity.objects.filter(entity_type=ContentType.objects.get_for_model(Account)).count(), 5)
+        self.assertEquals(Entity.objects.filter(entity_type=ContentType.objects.get_for_model(Team)).count(), 2)
+        self.assertEquals(Entity.objects.all().count(), 7)
+
+        # There should be four entity relationships since four accounts have teams
+        self.assertEquals(EntityRelationship.objects.all().count(), 4)
